#!/usr/bin/perl -w

# admin.cgi: submit recommendation letter for Faculty applicant
#            and also generate letter for acknowledgement and Affirmative Action Form
#            and also option to remove withdrawals and duplicates
#            and various other administrative functions
#
# Copyright (C) 2000-2003 Filippo Menczer and the University of Iowa
# Copyright (C) 2003-2004 Filippo Menczer and Indiana University 
#
#    This file is part of the Recruit system
#
#    Recruit is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Recruit is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Recruit; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# CONVENTIONS: 
# - shared admin password (encrypted) is stored in file 'passwd'
# - applicant's info is stored in database $ID/$ID.DB
# - applicant's recommendation letter X is stored in file $path/$ID/$ID-letter-X.(txt|pdf)
# - applicant's CV is stored in file $path/$ID/$ID-cvfile.pfd
# - applicant's statement is stored in file $path/$ID/$ID-stmtfile.pfd
# - evaluations (intersection table) are stored in 'eval.DB'#

$|=1;
use strict;
use CGI::Carp qw(fatalsToBrowser);
use CGI qw(:standard);
use DB_File;

# PARAMETERS OF THE SEARCH
use lib qw(.);
use params ();
use vars qw(%global);
*global = \%params::global;
my $dept = $global{dept};
my $university = $global{university};
my $chair = $global{chair};
my $path = $global{path};
my $URLpath = $global{URLpath};
my $arealist = $global{areas};
my $passwdflag = $global{passwdflag};

# links for end page
sub print_next {
	print "You may go back to the ", a({href=>"admin.cgi"}, "admin"), 
	" script or return to the Recruit ", a({href=>"../index.cgi"}, "home"), ".\n";
}

# upload and save letter
sub upload_letter {
	
	 # get ID and reference number
	 my $ID = param('ID');
	 my $reference = param('ref');

	 # upload and save the letter 
	 if (param('letterfile')) { # PDF file
		  my $handle = param('letterfile');
		  my $first_line = <$handle>;
		  die "Only PDF uploads accepted -" unless ($first_line =~ /^%PDF/);
		  seek($handle, 0, 0) || die "cannot rewind file: $!"; # rewind after checking first line for format
		  my $filename = "$path/$ID/$ID-letter-$reference.pdf";
		  open(F, ">$filename") || die "cannot create $filename: $!";
		  my @file = <$handle>;
		  print F @file;
		  close F;
	 } elsif (param('letterascii')) { # ASCII text
		  my $text = param('letterascii');
		  $text =~ s/\n/\n<p>/gs;
		  my $filename = "$path/$ID/$ID-letter-$reference.txt";
		  open(F, ">$filename") || die "cannot create $filename: $!";
		  print F "<html>\n", $text, "\n</html>";
		  close F;
	 } else { # ERROR
		  die "cannot find either upload file or pasted text for applicant $ID letter.\n";
	 }

	 # ACK CLERK
	 print h2("Thank you, a letter for applicant $ID has been saved.");	
}

# update candidate records
sub enter_file_changes {
	
	 # get ID and reference number
	 my $ID = param('ID');

	 # update records
	 my @parameters = qw(
		  fname lname 
		  url email fax phone 
		  address affil school dept 
		  phdmonth phdyear 
		  ref1title ref1fname ref1lname ref1affil ref1email ref1fax ref1phone ref1note
		  ref2title ref2fname ref2lname ref2affil ref2email ref2fax ref2phone ref2note
		  ref3title ref3fname ref3lname ref3affil ref3email ref3fax ref3phone ref3note
		  ref4title ref4fname ref4lname ref4affil ref4email ref4fax ref4phone ref4note
		  ref5title ref5fname ref5lname ref5affil ref5email ref5fax ref5phone ref5note
		  ref6title ref6fname ref6lname ref6affil ref6email ref6fax ref6phone ref6note
	 );
	 my %cand;
	 tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	 $cand{'areas'} = join(', ', param('areas'));
	 foreach my $p (@parameters) {
		$cand{$p} = param($p);
	 }
	 untie %cand;
	 if (param('cvfile')) {
		 my $handle = param('cvfile');
		 my $first_cv = <$handle>;
		 die "Only PDF documents accepted -" unless $first_cv =~ /^%PDF/;
		 seek($handle, 0, 0) || die "cannot rewind file: $!"; 
		 my $filename = "$path/$ID/$ID-cvfile.pdf";
		 open(F, ">$filename") || die "cannot create $filename: $!";
		 my @file = <$handle>;
		 print F @file;
		 close F;
	 }
	 if (param('stmtfile')) {
		 my $handle = param('stmtfile');
		 my $first_st = <$handle>;
		 die "Only PDF documents accepted -" unless $first_st =~ /^%PDF/;
		 seek($handle, 0, 0) || die "cannot rewind file: $!"; 
		 my $filename = "$path/$ID/$ID-stmtfile.pdf";
		 open(F, ">$filename") || die "cannot create $filename: $!";
		 my @file = <$handle>;
		 print F @file;
		 close F;
	 }

	 # ACK CLERK
	 print h2("Thank you, information for applicant $ID has been updated.");	
}

# change any field in applicant record
sub update_file($$) {
	 my($ID, $passwd) = @_;

	print start_multipart_form();
	my %cand;
	tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	print "<table>\n";	
	print Tr(td("First Name:"), td( textfield('fname',"$cand{'fname'}",50)));
	print Tr(td("Last Name:"), td( textfield('lname',"$cand{'lname'}",50)));
	print Tr(td("Email:"), td( textfield('email',"$cand{'email'}",50)));
	print Tr(td("Homepage (optional):"), td( textfield('url',"$cand{'url'}",50)));
	print Tr(td("Fax (optional):"), td( textfield('fax',"$cand{'fax'}",15)));
	print Tr(td("Phone (optional):"), td( textfield('phone',"$cand{'phone'}",15)));
	print Tr(td("Address:"), td( textarea('address',"$cand{'address'}",5,50)));
	print Tr(td("Current Affiliation:"), td( textarea('affil',"$cand{'affil'}",5,50)));
	print Tr(td("PhD Institution:"), td( textfield('school',"$cand{'school'}",50)));
	print Tr(td("PhD Department:"), td( textfield('dept',"$cand{'dept'}",50)));
	print Tr(td("PhD received/anticipated:"), 
		td("Month:" . textfield('phdmonth',"$cand{'phdmonth'}", 2, 2) . "Year:" . textfield('phdyear',"$cand{'phdyear'}", 5, 4)));
	my $cand_areas = $cand{'areas'};
	my @selectedareas;
	foreach my $area (@$arealist) {
		push(@selectedareas, $area) if $cand_areas =~ /$area/i;
	}
	print Tr(td("Research area(s):"), td( checkbox_group('areas', $arealist, \@selectedareas, 'true')));
	print Tr(td("Ref1 Title: "), td(textfield('ref1title',"$cand{'ref1title'}",4)));
	print Tr(td("Ref1 First Name: "), td(textfield('ref1fname',"$cand{'ref1fname'}",50))); 
	print Tr(td("Ref1 Last Name: "), td(textfield('ref1lname',"$cand{'ref1lname'}",50))); 
	print Tr(td("Ref1 Affiliation: "), td(textfield('ref1affil',"$cand{'ref1affil'}",50))); 
	print Tr(td("Ref1 Email: "), td(textfield('ref1email',"$cand{'ref1email'}",50)));
	print Tr(td("Ref1 Phone: "), td(textfield('ref1phone',"$cand{'ref1phone'}",15)));
	print Tr(td("Ref1 Fax: "), td(textfield('ref1fax',"$cand{'ref1fax'}",15)));
	print Tr(td("Ref1 Notes: "), td(textarea('ref1note',"$cand{'ref1note'}",3,50)));
	print Tr(td("Ref2 Title: "), td(textfield('ref2title',"$cand{'ref2title'}",4))); 
	print Tr(td("Ref2 First Name: "), td(textfield('ref2fname',"$cand{'ref2fname'}",50)));
	print Tr(td("Ref2 Last Name: "), td(textfield('ref2lname',"$cand{'ref2lname'}",50))); 
	print Tr(td("Ref2 Affiliation: "), td(textfield('ref2affil',"$cand{'ref2affil'}",50))); 
	print Tr(td("Ref2 Email: "), td(textfield('ref2email',"$cand{'ref2email'}",50)));
	print Tr(td("Ref2 Phone: "), td(textfield('ref2phone',"$cand{'ref2phone'}",15)));
	print Tr(td("Ref2 Fax: "), td(textfield('ref2fax',"$cand{'ref2fax'}",15)));
	print Tr(td("Ref2 Notes: "), td(textarea('ref2note',"$cand{'ref2note'}",3,50)));
	print Tr(td("Ref3 Title: "), td(textfield('ref3title',"$cand{'ref3title'}",4))); 
	print Tr(td("Ref3 First Name: "), td(textfield('ref3fname',"$cand{'ref3fname'}",50))); 
	print Tr(td("Ref3 Last Name: "), td(textfield('ref3lname',"$cand{'ref3lname'}",50))); 
	print Tr(td("Ref3 Affiliation: "), td(textfield('ref3affil',"$cand{'ref3affil'}",50))); 
	print Tr(td("Ref3 Email: "), td(textfield('ref3email',"$cand{'ref3email'}",50)));
	print Tr(td("Ref3 Phone: "), td(textfield('ref3phone',"$cand{'ref3phone'}",15)));
	print Tr(td("Ref3 Fax: "), td(textfield('ref3fax',"$cand{'ref3fax'}",15)));
	print Tr(td("Ref3 Notes: "), td(textarea('ref3note',"$cand{'ref3note'}",3,50)));
	print Tr(td("Ref4 Title: "), td(textfield('ref4title',"$cand{'ref4title'}",4))); 
	print Tr(td("Ref4 First Name: "), td(textfield('ref4fname',"$cand{'ref4fname'}",50))); 
	print Tr(td("Ref4 Last Name: "), td(textfield('ref4lname',"$cand{'ref4lname'}",50))); 
	print Tr(td("Ref4 Affiliation: "), td(textfield('ref4affil',"$cand{'ref4affil'}",50))); 
	print Tr(td("Ref4 Email: "), td(textfield('ref4email',"$cand{'ref4email'}",50))); 
	print Tr(td("Ref4 Phone: "), td(textfield('ref4phone',"$cand{'ref4phone'}",15)));
	print Tr(td("Ref4 Fax: "), td(textfield('ref4fax',"$cand{'ref4fax'}",15)));
	print Tr(td("Ref4 Notes: "), td(textarea('ref4note',"$cand{'ref4note'}",3,50)));
	print Tr(td("Ref5 Title: "), td(textfield('ref5title',"$cand{'ref5title'}",4))); 
	print Tr(td("Ref5 First Name: "), td(textfield('ref5fname',"$cand{'ref5fname'}",50))); 
	print Tr(td("Ref5 Last Name: "), td(textfield('ref5lname',"$cand{'ref5lname'}",50))); 
	print Tr(td("Ref5 Affiliation: "), td(textfield('ref5affil',"$cand{'ref5affil'}",50))); 
	print Tr(td("Ref5 Email: "), td(textfield('ref5email',"$cand{'ref5email'}",50))); 
	print Tr(td("Ref5 Phone: "), td(textfield('ref5phone',"$cand{'ref5phone'}",15)));
	print Tr(td("Ref5 Fax: "), td(textfield('ref5fax',"$cand{'ref5fax'}",15)));
	print Tr(td("Ref5 Notes: "), td(textarea('ref5note',"$cand{'ref5note'}",3,50)));
	print Tr(td("Ref6 Title: "), td(textfield('ref6title',"$cand{'ref6title'}",4))); 
	print Tr(td("Ref6 First Name: "), td(textfield('ref6fname',"$cand{'ref6fname'}",50))); 
	print Tr(td("Ref6 Last Name: "), td(textfield('ref6lname',"$cand{'ref6lname'}",50))); 
	print Tr(td("Ref6 Affiliation: "), td(textfield('ref6affil',"$cand{'ref6affil'}",50))); 
	print Tr(td("Ref6 Email: "), td(textfield('ref6email',"$cand{'ref6email'}",50))); 
	print Tr(td("Ref6 Phone: "), td(textfield('ref6phone',"$cand{'ref6phone'}",15)));
	print Tr(td("Ref6 Fax: "), td(textfield('ref6fax',"$cand{'ref6fax'}",15)));
	print Tr(td("Ref6 Notes: "), td(textarea('ref6note',"$cand{'ref6note'}",3,50)));
	print Tr(td("CV file: "), td(filefield('cvfile','',40)));
	print Tr(td("Stmt file: "), td(filefield('stmtfile','',40)));
	print "</table>\n";
	untie %cand;
	 
	 # save ID, stage and passwd in hidden fields
	 print hidden('ID', $ID);
	 print hidden('secret', $passwd);
	 param('stage',3);
	 print hidden('stage',3);

	 # set this so in following stage we know what to do
	 print hidden('update_task', 'file');

	 print p(submit("Submit Changes!"), reset("Clear Form"));
	 print end_form();
}

# directories, named 0,1,...
sub get_dirs {
	my @dirs;
	opendir(DIR, ".");
	while (defined($_ = readdir(DIR))) {
		if ((-d $_) && (/^(\d+)$/)) {
			push @dirs, $1;
		}
	}
	closedir(DIR);
	return sort {$a <=> $b} @dirs;
}

# check password against stored encrypted one (make sure file 'passwd' is there!)
# NB: if $passwdflag==0, then the check is passed automatically without any password!
# NB: hope salt is OK ...
sub check($) {
	return 1 unless $passwdflag; ### NO AUTHENTICATION!!!!
	my $clear = shift;
	return 0 unless defined($clear);
	open(PW, "passwd") || die "passwd file not found: $!";
	my $encoded = <PW>;
	close PW;
	chomp($encoded);
	if (crypt($clear, $encoded) eq $encoded) {
		return 1;
	} else {
		return 0;
	}
}

# input recomm letter
sub add_letter($$) {
	 my($ID, $passwd) = @_;
	 print start_multipart_form();
    
	 # determine which letter it is (start with refs)
	 my %cand;
	 tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	 my %label;	
	 my $rno;
	 foreach (1..6) {
		 my $rnm = 'ref'.$_.'lname';
		 if (length $cand{$rnm}) {
			 $label{$_} = $_.". ".$cand{$rnm};
			 $rno = $_;
		 }
	 }
	 
	 # check for existing letters and figure out
	 # appropriate next ref number for potential extra letter
	 opendir(DIR, "$path/$ID");
	 my @letters;
	 while (defined($_ = readdir(DIR))) {
	 	if (/^$ID-letter-(\d)\./) {
			unless (exists $label{$1}) {
				$label{$1} = "$1. Extra";
				$rno = $1 if $1 > $rno;
			}
			push @letters, $label{$1};
	 	}
	 }
	 closedir(DIR);
	 $rno++;	 
	 $label{$rno} = $rno.'. NEW EXTRA LETTER';
	 my @refnos = sort keys %label;
	 
	 # warn of existing letters
	 print p("<strong>Either</strong> upload a PDF file, <strong>or</strong> copy and paste letter (text only) into the text area below. File takes precedence. ");
	 if (scalar(@letters)) {
		  my $warn = "WARNING! The following letters are already present and would be replaced: ";
		  $warn .= "<strong>" . join(', ', sort @letters) . "</strong>";
		  print p($warn);
	 }
	 
	 # pick the letter
	 print p("Choose reference for applicant $ID ($cand{'fname'} $cand{'lname'}):", 
		 popup_menu('ref', \@refnos, '1', \%label));
	 untie %cand;
	 
	 # upload the letter
	 print filefield('letterfile','',60);
	 print p, textarea('letterascii','',15,60);
	 
	 # save ID, stage and passwd in hidden fields
	 print hidden('ID', $ID);
	 print hidden('secret', $passwd);
	 param('stage',3);
 	 print hidden('stage',3);
	 
	 # set this so in following stage we know what to do
	 print hidden('update_task', 'letter');

	 print p(submit("Submit Letter!"), reset("Clear Form"));
 	 print end_form();
}

# generate archive of file and download for printing
sub download_archive($) {
	 my $ID = shift;
	
	 # create temporary directory
	 mkdir("$ID/$ID-archive", 0750) or die "cannot create temp directory: $!";

	 # read candidate database and create index file
	 my %cand;
	 tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	 open(OUT, ">$ID/$ID-archive/index.html") or die "cannot write index to temp directory: $!";
	 print (OUT start_html(), h1("$dept Recruit: Application file $ID"), "\n");
	 print (OUT 'First Name: ', $cand{'fname'}, "<br>\n");
	 print (OUT 'Last Name: ', $cand{'lname'}, "<br>\n");
	 print (OUT 'Application Date: ', $cand{'date'}, "<br>\n");
	 print (OUT 'Homepage: ', $cand{'url'}, "<br>\n");
	 print (OUT 'Email: ', $cand{'email'}, "<br>\n");
	 print (OUT 'Fax: ', $cand{'fax'}, "<br>\n");
	 print (OUT 'Phone: ', $cand{'phone'}, "<br>\n");
	 print (OUT 'Address: ', $cand{'address'}, "<br>\n");
	 print (OUT 'Affiliation: ', $cand{'affil'}, "<br>\n");
	 print (OUT 'PhD Degree: ', $cand{'dept'}, "<br>\n");
	 print (OUT 'PhD Institution: ', $cand{'school'}, "<br>\n");
	 print (OUT 'PhD Date: ', "$cand{'phdmonth'}/$cand{'phdyear'}", "<br>\n");
	 print (OUT 'Areas: ', $cand{'areas'}, "<br>\n");
	 foreach my $refno (1..6) {
		 my $emailkey = $cand{'ref'."$refno".'email'};
		 my $lnamekey = $cand{'ref'."$refno".'lname'};
		 next unless defined $emailkey && defined $lnamekey;
		 my $affilkey = $cand{'ref'."$refno".'affil'};
		 my $titlekey = $cand{'ref'."$refno".'title'};
		 my $fnamekey = $cand{'ref'."$refno".'fname'};
		 my $phonekey = $cand{'ref'."$refno".'phone'};
		 my $faxkey = $cand{'ref'."$refno".'fax'};
		 my $notekey = $cand{'ref'."$refno".'note'};
		 print(OUT p("Reference $refno: $titlekey $fnamekey $lnamekey, ",
			"Affiliation:  $affilkey, Email: $emailkey, ", 
			defined $phonekey ? "Phone: $phonekey, ":'', 
			defined $faxkey ? "Fax: $faxkey, ":'', 
			defined $notekey ? "Note: $notekey":'None'), "\n");
	 }
	 print (OUT hr(), "&copy; 2000-2004 ", a({href=>"http://www.informatics.indiana.edu/fil/", target=>"_new"}, "Filippo Menczer"));
	 print (OUT end_html());
	 close OUT;
	 untie %cand;
	 
	 # copy docs (letters, cv and statement)
	 opendir(DIR, "$path/$ID");
	 while (defined($_ = readdir(DIR))) {
		if (/\.txt$/) {		 
			system("cp $path/$ID/$_ $ID/$ID-archive/$_.html");
		} elsif (/\.pdf$/) {
			system("cp $path/$ID/$_ $ID/$ID-archive");
		}
	 }
	 closedir(DIR);
	 
	 # create the TGZ archive and link for user to download
	 chdir("$ID");
	 system("tar czf $path/$ID/$ID.tgz $ID-archive");
	 print "Download ", 
		a({href=>"$URLpath/$ID/$ID.tgz", target=>"_new"}, "archive of candidate $ID"),
		" for printing.";
	 
	 # delete temp directory
	 opendir(DIR, "$ID/$ID-archive");
	 while (defined($_ = readdir(DIR))) {
		 unlink "$ID/$ID-archive/$_" unless ($_ eq '.' || $_ eq '..');
	 }
	 closedir(DIR);
	 rmdir "$ID/$ID-archive";
}

# generate ack letter
sub acknowledge($) {
	 my $ID = shift;
	 print h2("Thank you for updating application $ID, which is no longer marked as NEW.");
	 print p("You may copy the letter text below and paste into your email or word processor."), hr;
	 
	 # get applicant info and set status	
	 my %cand;
	 tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	 $cand{'new'} = "Review";
	 
	 # print letter text
	 my ($day, $month, $year) = (localtime)[3,4,5];
	 $month++; $year+=1900;
	 print tt(
		  "$month/$day/$year", p,
		  "$cand{'fname'} $cand{'lname'}", br,
		  $cand{'address'}, p,
		  "Dear $cand{'fname'} $cand{'lname'}:", p,
		  "Thank you for submitting an application for a position in the $dept ",
		  "at $university.  The Recruiting Committee is currently reviewing applications.", p,
		  "An Affirmative Action Form and return envelope is enclosed.  We would appreciate it ",
		  "if you would take a moment to fill it out and return it to Affirmative Action ",
		  "in the enclosed envelope.", p,
		  "We wish you success in your job search this year.", p,
		  "Sincerely,", p, "&nbsp", p,
		  $chair, br,
		  "Chair, Search Committee", p,
		  "Encl."
	 );
	 
	 # done
	 untie %cand;
}

# change status of applicant
sub set_status($$) {
	 my ($ID, $status) = @_;
	 
	 unless (length $status) {
		 print p("Status of application $ID was not changed (empty status string).");		 
		 return;		 
	 }
	 
	 # get applicant info and set status	
	 my %cand;
	 tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	 $cand{'new'} = $status;
	 
	 # done
	 untie %cand;
	 print p("Status of application $ID was changed to <$status>. Thank you.");	 
	 print_next();
}

# generate rejection letter
sub reject($) {
	 my $ID = shift;
	 print h2("Thank you for updating application $ID, which is now marked as REJECTED.");
	 print p("You may copy the letter text below and paste into your email or word processor. ",
	 "Choose the appropriate text depending on the candidate's status: ", 
     a({href=>'#before'}, 'before'), 'or', a({href=>'#after'}, 'after'), "interview.");
	 
	 # get applicant info and set status	
	 my %cand;
	 tie(%cand, 'DB_File', "$ID/$ID.DB") || die "Cannot open candidate $ID DB_File: $!";
	 $cand{'new'} = "Reject";
	 my ($day, $month, $year) = (localtime)[3,4,5];
	 $month++; $year+=1900;
	 
	 # print letter text BEFORE INTERVIEW
	 print hr;
	 print a({name=>'before'}), "\n";
	 print h3('Use text below for rejection BEFORE interview:');
	 print hr;
	 print tt(
		  "$month/$day/$year", p,
		  "$cand{'fname'} $cand{'lname'}", br,
		  $cand{'address'}, p,
		  "Dear $cand{'fname'} $cand{'lname'}:", p,
		  "Thank you for submitting an application for a position in the $dept ",
		  "at $university.  The Recruiting Committee has now reviewed your application.", p,
		  "I regret to report that you are not among the candidates selected for further consideration. ",
		  "Please realize that this decision is based on many factors, which include the match to our ",
		  "focus areas and programmatic needs, as well as the fiercely competitive academic job market.", p,
		  "We wish you every success in your job search.", p,
		  "Sincerely,", p, "&nbsp;", p,
		  $chair, br,
		  "Chair, Search Committee"
	 );
	 
	 # print letter text AFTER INTERVIEW
	 print hr;
	 print a({name=>'after'}), "\n";
	 print h3('Use text below for rejection AFTER interview:');
	 print hr;
	 print tt(
		  "$month/$day/$year", p,
		  "$cand{'fname'} $cand{'lname'}", br,
		  $cand{'address'}, p,
		  "Dear $cand{'fname'} $cand{'lname'}:", p,
		  "The $dept faculty appreciated the opportunity to meet you ",
		  "during your interview at $university for a faculty position in the ",
		  "area of ", $cand{'areas'}, ".  Your visit gave us an opportunity to better understand your ", 
		  "research and to gain a sense of your teaching abilities, as well as to get ", 
		  "to know you as a potential colleague.", p,
		  "Although you have a strong record, and there were several positive comments ",
		  "about your visit, we regret to inform you that we have withdrawn your ",
		  "application for further consideration.", p,
		  "We truly thank you for your application and time, and we wish you every ",
		  "success in your job search.", p,
		  "Warm regards,", p, "&nbsp;", p,
		  $chair, br,
		  "Chair, Search Committee"
	 );
	 
	 # done
	 untie %cand;
}

# remove an application, such as a duplicate
sub remove_dup($) {
	my $ID = shift;
	 
	# applicant's database
	unlink "$ID/$ID.DB";
	rmdir "$ID";
	
	# applicant's docs and letters
	opendir(DIR, "$path/$ID");
	while (defined($_ = readdir(DIR))) {
		unlink "$path/$ID/$_" unless ($_ eq '.' || $_ eq '..');
	}
	closedir(DIR);
	rmdir "$path/$ID";
	
	# evaluations (intersection table) key format: 'rID' $; 'aID'
	my %evals;
	tie(%evals, 'DB_File', "eval.DB") || die "Cannot open eval DB_File: $!";
	my $k;
	foreach $k (keys(%evals)) {
		if ($k =~ /$;$ID$/) {
			delete $evals{$k};
		}
	}
	untie %evals;
	 
	# done
	print h2("Application $ID has been deleted.");
	print_next();	 
}

# MAIN
if (!param() && !check(cookie('cypher'))) { # stage 0, enter password
	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Update file for faculty applicant in $dept");
    print start_form();
	 print p("Password:", password_field('secret','',50), submit("Submit!"));
	 print hidden('stage',1);
    print end_form();
	 
} elsif ((!param() && check(cookie('cypher'))) || (param('stage') == 1 && check(param('secret')))) { # stage 1

	# set the cookie for authenticated user
	my $passwd;
	if (!$passwdflag) {
		$passwd = ''; # does not matter what as long as defined
	} elsif (defined(param('secret'))) { # user just entered password
		$passwd = param('secret');
		print "Set-Cookie: cypher=$passwd\n"; # set the cookie
	} else { # password from cookie
		$passwd = cookie('cypher');
	}
	
	# start page
	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Update file for faculty applicant in $dept");
    print start_form();

    # select applicant 
	print p("Pick an applicant:");
    my @apps = get_dirs();
    my %labels;
	 foreach my $id (@apps) { # create %labels
		my %cand;
		tie(%cand, 'DB_File', "$id/$id.DB") || die "Cannot open candidate $id DB_File: $!";
		$labels{$id} = "$id. $cand{'lname'}, $cand{'fname'}";
		untie %cand;
 	 }
	 print scrolling_list('candid', \@apps, [], 10, 'false', \%labels);
	 
	 # decide what to do
	 print p, radio_group('option', [
		'Generate Acknowledgement Letter',
		'Enter Recommendation Letter',
		'Update Application', 
		'Download File To Disk', 
		'Generate Rejection Letter',
		'Remove Duplicate',
		'Set Status To:' 
		 ], 'Generate Acknowledgement Letter', 'true'),
		textfield('newstatus', '', 16, 64), 
		'(for other actions, eg letters requested, interview, etc)';

	 # save stage and passwd in hidden fields
	 param('stage',2);
 	 print hidden('stage',2);
	 print hidden('secret', $passwd);

	 print p(submit("Go!"));
     print end_form();
    
} elsif (param('stage') == 2 && check(param('secret'))) { # stage 2

	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Update file for faculty applicant in $dept");
	 
	 # determine applicant and then perform appropriate task	
	 if (param('option') eq 'Enter Recommendation Letter') {
	 	 add_letter(param('candid'),param('secret'));
	 } elsif (param('option') eq 'Update Application') {
		 update_file(param('candid'),param('secret'));
	 } elsif (param('option') eq 'Generate Acknowledgement Letter') {
		  acknowledge(param('candid'));
	 } elsif (param('option') eq 'Download File To Disk') {
		  download_archive(param('candid'));
	 } elsif (param('option') eq 'Generate Rejection Letter') {
		   reject(param('candid'));
	 } elsif (param('option') eq 'Remove Duplicate') {
		  remove_dup(param('candid'));
	 } elsif (param('option') eq 'Set Status To:') {
		  set_status(param('candid'), param('newstatus'));
	 } else {
		  warn "Error: unrecognized option!";
	 }
	 	 
} elsif (param('stage') == 3 && check(param('secret'))) { # stage 3

	print header, start_html(-title=>"$dept Recruit"), 
		h1("Update file for faculty applicant in $dept");

	if (param('update_task') eq 'letter') {
		upload_letter();	
	} elsif (param('update_task') eq 'file') {
		enter_file_changes();
	} else {
		die "unrecognized update task option\n";
	}
	   
	print_next();

} else { # FAILURE STAGE

	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Update file for faculty applicant in $dept");
  	 print h2("Authentication failed.");
  	 
}
print hr, a({href=>"http://www.informatics.indiana.edu/fil/Recruit/", target=>"_new"}, 'Recruit'), " &copy; 2000-2004 ", a({href=>"http://www.informatics.indiana.edu/fil/", target=>"_new"}, 'Filippo Menczer'); 
print end_html;

